@@ -26,10 +26,10 @@ let get_elpi_code_appArg = ref (fun _ -> assert false)
 
 let get_ctx, set_ctx, _update_ctx =
   let bound_vars =
-    S.declare ~name:"coq-elpi:glob-quotation-bound-vars"
+    S.declare_component ~name:"coq-elpi:glob-quotation-bound-vars" ~descriptor:interp_state
       ~init:(fun () -> None)
       ~pp:(fun fmt -> function Some (x,_) -> () | None -> ())
-      ~start:(fun x -> x)
+      ~start:(fun x -> x) ()
        in
   S.(get bound_vars, set bound_vars, update bound_vars)
 
@@ -59,17 +59,22 @@ let is_restricted_name =
   fun s -> Str.(string_match rex (Id.to_string s) 0)
 
 
-let glob_environment : Environ.env S.component =
-  S.declare ~name:"coq-elpi:glob-environment"
-    ~pp:(fun _ _ -> ()) ~init:Global.env ~start:(fun _ -> Global.env ())
+let glob_environment : Environ.env option S.component =
+  S.declare_component ~name:"coq-elpi:glob-environment" ~descriptor:interp_state
+    ~pp:(fun _ _ -> ()) ~init:(fun () -> None)
+                        ~start:(fun _ -> Some (Global.env ())) ()
+
+(* Since Accumulate runs before the interpreter starts, the state
+   may be empty: ~start not called, ~init called too early *)
+let ensure_some f = function None -> Some (f (Global.env ())) | Some x -> Some (f x)
 
 let push_env state name =
   let open Context.Rel.Declaration in
-  S.update glob_environment state (Environ.push_rel (LocalAssum(Context.make_annot name Sorts.Relevant,Constr.mkProp)))
+  S.update glob_environment state (ensure_some (Environ.push_rel (LocalAssum(Context.make_annot name Sorts.Relevant,Constr.mkProp))))
 let pop_env state =
-  S.update glob_environment state (Environ.pop_rel_context 1)
+  S.update glob_environment state (ensure_some (Environ.pop_rel_context 1))
   
-let get_glob_env state = S.get glob_environment state
+let get_glob_env state = Option.get @@ ensure_some (fun x -> x) @@ S.get glob_environment state
 
 (* XXX: I don't get why we use a coq_ctx here *)
 let under_ctx name ty bo gterm2lp ~depth state x =
@@ -152,9 +157,10 @@ let rec gterm2lp ~depth state x =
       let state, f = F.Elpi.make state in
       let s = API.RawData.mkUnifVar f ~args:[] state in
       state, in_elpi_poly_gr ~depth state gr s
-    | Some l -> 
+    | Some (ql,l) ->
+      let () = if not (CList.is_empty ql) then nYI "sort poly" in
       let l' = List.map (glob_level x.CAst.loc state) l in
-      state, in_elpi_poly_gr_instance ~depth state gr (Univ.Instance.of_array (Array.of_list l'))
+      state, in_elpi_poly_gr_instance ~depth state gr (UVars.Instance.of_array ([||], Array.of_list l'))
     end
   | GRef(gr,_ul) -> state, in_elpi_gr ~depth state gr
   | GVar(id) ->
@@ -164,7 +170,7 @@ let rec gterm2lp ~depth state x =
           Pp.(str"Free Coq variable " ++ Names.Id.print id ++ str " in context: " ++
             prlist_with_sep spc Id.print (Id.Map.bindings ctx.name2db |> List.map fst));
       state, E.mkConst (Id.Map.find id ctx.name2db)
-  | GSort(UAnonymous {rigid=true}) ->
+  | GSort(UAnonymous {rigid=UnivRigid}) ->
       let state, f = F.Elpi.make state in
       let s = API.RawData.mkUnifVar f ~args:[] state in
       state, in_elpi_flex_sort s
@@ -300,7 +306,7 @@ let rec gterm2lp ~depth state x =
          * the term can be read back (mkCases needs the ind) *)
         (* TODO: add whd reduction in spine *)
         let ty =
-          Inductive.type_of_inductive (indspecif,Univ.Instance.empty) in
+          Inductive.type_of_inductive (indspecif,UVars.Instance.empty) in
         let safe_tail = function [] -> [] | _ :: x -> x in
         let best_name n l = match n, l with
           | _, (Name x) :: rest -> Name x,DAst.make (GVar x), rest
@@ -394,16 +400,15 @@ let coq_quotation ~depth state loc src =
       Constrintern.intern_constr (get_glob_env state) (get_sigma state) ce
     with e ->
       CErrors.user_err
-        Pp.(str(API.Ast.Loc.show loc) ++str":" ++ spc() ++ CErrors.print_no_report e)
+        Pp.(str(API.Ast.Loc.show loc) ++ spc() ++ CErrors.print_no_report e)
   in
   gterm2lp ~depth state glob
       
 
 (* Install the quotation *)
-let () = Q.set_default_quotation coq_quotation
-let () = Q.register_named_quotation ~name:"coq" coq_quotation
-
-let () = API.Quotation.register_named_quotation ~name:"gref"
+let () = Q.set_default_quotation coq_quotation ~descriptor:interp_quotations
+let () = Q.register_named_quotation ~name:"coq" coq_quotation ~descriptor:interp_quotations
+let () = API.Quotation.register_named_quotation ~name:"gref" ~descriptor:interp_quotations
   (fun ~depth state _loc src ->
     let gr = locate_gref src in
     let state, gr, gls = gref.API.Conversion.embed ~depth state gr in
